﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Threading;

namespace ProducerConsumerBC
{
    class Program
    {
        static void Main(string[] args)
        {
            var blockingCollection = new BlockingCollection<int>(10);
            List<Thread> threads = new List<Thread>();
           
            //the producer puts 100 elements into the collection at random intervals of up to 500ms
            var producer = new Thread(() =>
            {
                Random rand = new Random(Thread.CurrentThread.ManagedThreadId);
                for (int l = 0; l < 100; l++)
                {
                    Console.WriteLine($"Put  {l}");
                    blockingCollection.Add(l);
                    Thread.Sleep(rand.Next(500));
                }
                blockingCollection.CompleteAdding();
            });
            producer.Start();
            Thread.Sleep(5000); //let's wait a moment, then we will observe the effect of filling the collection. 
            //The producer doesn't put anything on anymore. 
            //Consumers taking data from the collection at intervals of up to 5000 ms
            for (int i = 0; i < 10; i++)
            {
                var consumer = new Thread(() =>
                {
                    Random rand = new Random(Thread.CurrentThread.ManagedThreadId);
                    //  bool collectionReady;
                    for (int l = 0; l < 10; l++)
                    {
                        // collectionReady = blockingCollection.TryTake(out var result2); //what happens when we use a non-blocking function?
                        var result2 = blockingCollection.Take(); //when there is no data, we do not take it.
                        Console.WriteLine($"Take {result2} ");
                        Thread.Sleep(rand.Next(5000));
                    }
                });
                threads.Add(consumer);
                consumer.Start();
            }
            foreach (var th in threads)
            {
                th.Join();
            }
            producer.Join();
            Console.WriteLine("Finished");
            Console.ReadLine();
        }
    }
}
